// compute result in a series of sequential steps
module fulladd_procedural 
   ( input logic Aa_DI, Bb_DI, Cc_DI, 
     output logic Sum_DO, Carry_DO );

   always_comb
   begin 
      logic loc1, loc3, loc4;
      loc1     =   Aa_DI ^ Bb_DI;
      Sum_DO   =   Cc_DI ^ loc1;
      loc3     = ~(Cc_DI & loc1);
      loc4     = ~(Aa_DI & Bb_DI);
      Carry_DO = ~(loc3  & loc4);
    end 
endmodule
//-----------------------------------------------------------------------------
// spawn a continuous assignment for each logic operation 
module fulladd_dataflow
   ( input logic Aa_DI, Bb_DI, Cc_DI, 
     output logic Sum_DO, Carry_DO );

   logic Loc1_D, Loc3_D, Loc4_D;
   assign Loc1_D   =   Aa_DI  ^ Bb_DI;
   assign Sum_DO   =   Cc_DI  ^ Loc1_D;
   assign Loc3_D   = ~(Cc_DI  & Loc1_D);
   assign Loc4_D   = ~(Aa_DI  & Bb_DI);
   assign Carry_DO = ~(Loc3_D & Loc4_D);
endmodule
//-----------------------------------------------------------------------------
// describe circuit network as a bunch of interconnected std cells
// note: cells from Synopsys' generic library are used here
module fulladd_structuralgtech
   ( input logic Aa_DI, Bb_DI, Cc_DI, 
     output logic Sum_DO, Carry_DO );

   logic Loc1_D, Loc3_D, Loc4_D;
   GTECH_XOR2  u1 ( .A(Bb_DI),  .B(Aa_DI),  .Z(Loc1_D) );
   GTECH_XOR2  u2 ( .A(Cc_DI),  .B(Loc1_D), .Z(Sum_DO) );
   GTECH_NAND2 u3 ( .A(Cc_DI),  .B(Loc1_D), .Z(Loc3_D) );
   GTECH_NAND2 u4 ( .A(Aa_DI),  .B(Bb_DI),  .Z(Loc4_D) );
   GTECH_NAND2 u5 ( .A(Loc3_D), .B(Loc4_D), .Z(Carry_DO) );
endmodule
