entity fulladd is
   port (
      Aa_DI, Bb_DI, Cc_DI : in std_logic;
      Sum_DO, Carry_DO : out std_logic );
end fulladd;
-------------------------------------------------------------------------------
-- compute result in a series of sequential steps
architecture procedural1 of fulladd is
begin
   process (Aa_DI,Bb_DI,Cc_DI)
      variable loc1, loc3, loc4 : std_logic;
   begin
      loc1     := Aa_DI xor  Bb_DI;
      Sum_DO   <= Cc_DI xor  loc1;
      loc3     := Cc_DI nand loc1;
      loc4     := Aa_DI nand Bb_DI;
      Carry_DO <= loc3  nand loc4;
   end process;
end procedural1;
-------------------------------------------------------------------------------
-- spawn a concurrent signal assignment for each logic operation
architecture dataflow1 of fulladd is
   signal Loc1_D, Loc3_D, Loc4_D : std_logic;
begin
   Loc1_D   <= Aa_DI  xor  Bb_DI;
   Sum_DO   <= Cc_DI  xor  Loc1_D;
   Loc3_D   <= Cc_DI  nand Loc1_D;
   Loc4_D   <= Aa_DI  nand Bb_DI;
   Carry_DO <= Loc3_D nand Loc4_D;
end dataflow1;
-------------------------------------------------------------------------------
-- describe circuit network as a bunch of interconnected std cells
-- note: cells from Synopsys' generic library are used here
architecture structuralgtech of fulladd is

    component GTECH_XOR2
       port ( A, B : in std_logic;
              Z : out std_logic );
    end component;
    component GTECH_NAND2
       port ( A, B : in std_logic;
              Z : out std_logic );
    end component;

    signal Loc1_D, Loc3_D, Loc4_D : std_logic;

begin
    u1: GTECH_XOR2
        port map ( A=>Bb_DI,  B=>Aa_DI,  Z=>Loc1_D );
    u2: GTECH_XOR2
        port map ( A=>Cc_DI,  B=>Loc1_D, Z=>Sum_DO );
    u3: GTECH_NAND2
        port map ( A=>Cc_DI,  B=>Loc1_D, Z=>Loc3_D );
    u4: GTECH_NAND2
        port map ( A=>Aa_DI,  B=>Bb_DI,  Z=>Loc4_D );
    u5: GTECH_NAND2
        port map ( A=>Loc3_D, B=>Loc4_D, Z=>Carry_DO );
end structuralgtech;
