// Mission: illustrate the use of packages in SystemVerilog.
// Functionality: Gray code <-> binary code conversion functions.
// Author: H.Kaeslin.
//-----------------------------------------------------------------------------

`include "../sourcecode/graydefs.sv"    // the word width to be used  

package grayconvPkg;

`ifndef GRAYWIDTH 
   `define GRAYWIDTH 2  // default word width for input and output 
`endif

//-----------------------------------------------------------------------------
//
//                         |     |     |     |     |
// binary to Gray          |--.  |--.  |--.  |--.  |          # of X-ops on
// conversion              |  `--X  `--X  `--X  `--X          longest paths
// for GRAYWIDTH=5         |     |     |     |     |          = 1
//                         v     v     v     v     v
// bit positions           4     3     2     1     0          X = XOR
//                         |     |     |     |     |
// Gray to binary          |  ,--X  ,--X  ,--X  ,--X          # of X-ops on
// conversion              |--'  |--'  |--'  |--'  |          longest path
// for GRAYWIDTH=5         |     |     |     |     |          = GRAYWIDTH-1
//                         v     v     v     v     v
//
//-----------------------------------------------------------------------------

   function automatic logic [`GRAYWIDTH-1:0] bin2gray 
      (input [`GRAYWIDTH-1:0] arg);
      bin2gray=arg;                         // initial assignment
      for (int i=0; i <=$size(arg)-2; i++)  // from 0 to GRAYWIDTH-2
         if (bin2gray[i+1])                 // if bit at [i+1] is 1
         bin2gray[i] = ~bin2gray[i];        // invert bit at [i]
   endfunction

   function automatic logic [`GRAYWIDTH-1:0] gray2bin 
      (input [`GRAYWIDTH-1:0] arg);
      gray2bin=arg;                         // initial assignment
      for (int i=$size(arg)-2; i >=0; i--)  // from GRAYWIDTH-2 downto 0
         if (gray2bin[i+1])                 // if bit at [i+1] is 1
         gray2bin[i] = ~gray2bin[i];        // invert bit at [i]
   endfunction

endpackage : grayconvPkg
