// external interface of module
module lfsr4 (
      input logic Clk_CI, Rst_RBI, Ena_SI,   // reset is active low 
      output logic Oup_DO );

// behavioral model for module 

   // declare internal variables
   logic [1:4] State_DP, State_DN; // for present and next state 

   // computation of next state using concatenation of bits
   assign State_DN = {(State_DP[3] ^ State_DP[4]), State_DP[1:3]};
   
   // updating of state 
   always_ff @(posedge Clk_CI, negedge Rst_RBI)
      // activities triggered by asynchronous reset
      if (~Rst_RBI)
         State_DP <= 4'b0001;
      // activities triggered by rising edge of clock
      else
         if (Ena_SI)
            State_DP <= State_DN; // admit next state into state register
   
   // updating of output
   assign Oup_DO = State_DP[4];

endmodule
