// module header with external interface
module lfsr4
   ( output logic Oup_DO, 
     input logic Clk_CI, Rst_RBI, Ena_SI ) ; // reset is active low 

// module body with circuit structure

   // declare a variable for each inner node
   logic [1:4] State_DP;
   logic n11, n21, n31, n41, n42;

   // instantiate cells and connect them by listing port maps
   // (cells are from Synopsys' generic library in this example) 
   GTECH_FD2 u10 ( .D(n11), .CP(Clk_CI), .CD(Rst_RBI), .Q(State_DP[1]) );
   GTECH_FD2 u20 ( .D(n21), .CP(Clk_CI), .CD(Rst_RBI), .Q(State_DP[2]) );
   GTECH_FD2 u30 ( .D(n31), .CP(Clk_CI), .CD(Rst_RBI), .Q(State_DP[3]) );
   GTECH_FD4 u40 ( .D(n41), .CP(Clk_CI), .SD(Rst_RBI), .Q(State_DP[4]) );
   GTECH_MUX2 u11 ( .A(State_DP[1]), .B(n42),         .S(Ena_SI), .Z(n11) );
   GTECH_MUX2 u21 ( .A(State_DP[2]), .B(State_DP[1]), .S(Ena_SI), .Z(n21) );   
   GTECH_MUX2 u31 ( .A(State_DP[3]), .B(State_DP[2]), .S(Ena_SI), .Z(n31) );   
   GTECH_MUX2 u41 ( .A(State_DP[4]), .B(State_DP[3]), .S(Ena_SI), .Z(n41) );
   GTECH_XOR2 u42 ( .A(State_DP[3]), .B(State_DP[4]),             .Z(n42) );

   // connect state bit of rightmost flip-flop to output port
   assign Oup_DO = State_DP[4];

endmodule
