-- architecture body
architecture structural of lfsr4 is

   -- declare the components to be used
   component GTECH_FD2    -- D-type flip-flop with reset
      port (
         D, CP, CD : in  std_logic; 
         Q : out std_logic );
   end component;
   component GTECH_FD4    -- D-type flip-flop with set
      port (
         D, CP, SD : in  std_logic;
         Q : out std_logic );
   end component;
   component GTECH_MUX2   -- 2-input multiplexer
      port (
         A, B, S : in std_logic;
         Z : out std_logic );
   end component;
   component GTECH_XOR2   -- 2-input XOR gate
      port (
         A, B: in std_logic;
         Z : out std_logic );
   end component;  

   -- declare a signal for each inner node
   signal State_DP : std_logic_vector(1 to 4);
   signal n11, n21, n31, n41, n42 : std_logic;

begin

   -- instantiate components and connect them by listing port maps
   u10 : GTECH_FD2 
      port map( D => n11, CP => Clk_CI, CD => Rst_RBI, Q => State_DP(1) );
   u20 : GTECH_FD2 
      port map( D => n21, CP => Clk_CI, CD => Rst_RBI, Q => State_DP(2) );
   u30 : GTECH_FD2 
      port map( D => n31, CP => Clk_CI, CD => Rst_RBI, Q => State_DP(3) );
   u40 : GTECH_FD4 
      port map( D => n41, CP => Clk_CI, SD => Rst_RBI, Q => State_DP(4) );
   u11 : GTECH_MUX2
      port map( A => State_DP(1), B => n42, S => Ena_SI, Z => n11 );
   u21 : GTECH_MUX2
      port map( A => State_DP(2), B => State_DP(1), S => Ena_SI, Z => n21 );   
   u31 : GTECH_MUX2
      port map( A => State_DP(3), B => State_DP(2), S => Ena_SI, Z => n31 );   
   u41 : GTECH_MUX2
      port map( A => State_DP(4), B => State_DP(3), S => Ena_SI, Z => n41 );
   u42 : GTECH_XOR2
      port map( A => State_DP(3), B => State_DP(4), Z => n42 );
   
   -- connect state bit of rightmost flip-flop to output port
   Oup_DO <= State_DP(4);

end structural;
